package com.hyp.web.controller.manager;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hyp.common.config.HypConfig;
import com.hyp.common.core.domain.AjaxResult;
import com.hyp.common.exception.ServiceException;
import com.hyp.common.utils.file.FileUploadUtils;
import com.hyp.common.utils.file.FileUtils;
import com.hyp.framework.config.ServerConfig;
import com.hyp.system.domain.RewFileRelation;
import com.hyp.system.service.RewFileRelationService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * packageName com.hyp.web.controller.manager
 *
 * @author wangxy
 * @version JDK 8
 * @className FileRelationManager
 * @date 2024/4/12
 * @description 文件
 */
@Component
public class FileRelationManager {

    @Resource
    private RewFileRelationService fileRelationService;


    @Resource
    private ServerConfig serverConfig;

    /**
     * 未通过
     *
     */
    private static final Integer FILE_STATE_WTG = 2;

    /**
     *
     *  删除文件
     * @return boolean
     */
    public  boolean delFile(String fileId){
         RewFileRelation fileRelation = fileRelationService.getById(fileId);
         if(Objects.isNull(fileRelation)){
             throw new ServiceException("文件不存在");
         }
        FileUtil.del(fileRelation.getFilePath());
        return  fileRelationService.removeById(fileId);

    }
    /**
     *
     *  材料审核
     * @param applyId
     * @param fileCode
     * @param fileState
     * @return boolean
     */
    public  boolean filePass( String applyId, String fileCode, Integer fileState){
        return fileRelationService.lambdaUpdate()
                .eq(RewFileRelation::getApplyId,applyId)
                .eq(RewFileRelation::getFileCode,fileCode)
                .set(RewFileRelation::getFileState,fileState)
                .update();


    }

    /**
     *
     *  材料审核
     * @param fileId
     * @param fileState
     * @return boolean
     */
    public  boolean pass( String fileId,  Integer fileState){
        return fileRelationService.lambdaUpdate()
                .eq(RewFileRelation::getFileId,fileId)
                .set(RewFileRelation::getFileState,fileState)
                .update();
    }

    /**
     *
     *  验证文件是否审查
     * @param applyId
     * @return com.hyp.common.core.domain.AjaxResult
     */
    public AjaxResult checkFile(String applyId){
         List<RewFileRelation> list = fileRelationService.lambdaQuery()
                 .eq(RewFileRelation::getApplyId, applyId)
                 .list();
         if(CollUtil.isNotEmpty(list)){
             for(RewFileRelation rewFileRelation : list){
                 if(Objects.equals(rewFileRelation.getFileState(),FILE_STATE_WTG) || Objects.isNull(rewFileRelation.getFileState())){
                     return AjaxResult.error("材料存在未审核情况，请检查");
                 }
             }
         }
        return AjaxResult.success();
    }

    /**
     * 
     *  单个保存
     * @param applyId
     * @param filename
     * @param fileCode
     * @param url
     * @param originalFilename
     * @return com.hyp.system.domain.RewFileRelation
     */
    @Transactional(rollbackFor = Exception.class)
    public RewFileRelation save(String applyId,String filename,String fileCode,String url,String originalFilename){
        RewFileRelation rewFileRelation = new RewFileRelation();
        rewFileRelation.setApplyId(applyId);
        rewFileRelation.setFileName(filename);
        rewFileRelation.setFileCode(fileCode);
        rewFileRelation.setFilePath(url);
        rewFileRelation.setRealName(originalFilename);
        fileRelationService.save(rewFileRelation);
        return rewFileRelation;
    }



    /**
     * 
     *  批量保存文件
     * @param files
     * @param applyId
     * @param filename
     * @param fileCode
     * @return java.util.List<com.hyp.system.domain.RewFileRelation>
     */
    @Transactional(rollbackFor = Exception.class)
    public List<RewFileRelation> saveBatch(List<MultipartFile> files,
                                          String applyId,
                                          String filename,
                                          String fileCode) throws IOException {
        List<RewFileRelation> fileRelations = new ArrayList<>();
        // 上传文件路径
        String filePath = HypConfig.getUploadPath();
        for (MultipartFile file : files) {
            if (!file.isEmpty()) {
                // 上传并返回新文件名称
                String fileNames = FileUploadUtils.upload(filePath, file);
                String url = serverConfig.getUrl() + fileNames;
                if(StringUtils.isBlank(filename)){
                    filename = fileNames;
                }
                RewFileRelation rewFileRelation = new RewFileRelation();
                rewFileRelation.setApplyId(applyId);
                rewFileRelation.setFileName(filename);
                rewFileRelation.setFileCode(fileCode);
                rewFileRelation.setFilePath(url);
                rewFileRelation.setRealName(FileUtils.getNameNotSuffix(file.getOriginalFilename()));
                fileRelations.add(rewFileRelation);
            }
        }
        fileRelationService.saveBatch(fileRelations);
        return fileRelations;
    }

    /**
     *
     *  获取文件路劲
     * @param fileId
     * @return java.lang.String
     */
    public String getFilePath(String fileId){
         RewFileRelation fileRelation = fileRelationService.getById(fileId);
         if(Objects.nonNull(fileRelation)){
             return  fileRelation.getFilePath();
         }
        return null;
    }

    /**
     *
     *  获取文件名
     * @param fileId
     * @return java.lang.String
     */
    public String getRealName(String fileId){
        RewFileRelation fileRelation = fileRelationService.getById(fileId);
        if(Objects.nonNull(fileRelation)){
            return  fileRelation.getRealName();
        }
        return null;
    }

    /**
     *
     *  文件列表啥
     * @param applyId
     * @param fileCode
     * @return java.util.List<com.hyp.system.domain.RewFileRelation>
     */
    public List<RewFileRelation> getFileInfo(String applyId,String fileCode){
        LambdaQueryWrapper<RewFileRelation> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RewFileRelation::getApplyId,applyId);
        queryWrapper.eq(Objects.nonNull(fileCode), RewFileRelation::getFileCode, fileCode);
        return fileRelationService.list(queryWrapper);
    }


    /**
     *
     *  文件列表啥
     * @return java.util.List<com.hyp.system.domain.RewFileRelation>
     */
    public RewFileRelation getRewFileRelation(String fileId){
        return fileRelationService.getById(fileId);
    }




}
